@@ -227,7 +227,7 @@ void standardize_and_get_residual(NumericVector &center, NumericVector &scale,
                                   int *p_keep_ptr, vector<int> &col_idx, //columns to keep, removing columns whose scale < 1e-6
                                   vector<double> &z, double *lambda_max_ptr,
                                   int *xmax_ptr, XPtr<BigMatrix> xMat, double *y, 
-                                  int *row_idx, double lambda_min, double alpha, int n, int p) {
+                                  int *row_idx, double alpha, int n, int p) {
   MatrixAccessor<double> xAcc(*xMat);
   double *xCol;
   double sum_xy, sum_y;
